****************************************************************************************************
*Replication dofile for analyses presented in: 
*Florencia Torche and Peter Rich (2016) "Declining Racial Stratification in Marriage Choices? 
*  Trends in Black/White Status Exchange in the United States, 1980 to 2010" 
*  Sociology of Race and Ethnicity. (http://sre.sagepub.com/content/early/2016/05/27/2332649216648464)
*  
****************************************************************************************************

****************************************************************************************************
*Setup
****************************************************************************************************
global dir "[directory path]" // user: set directory path as location of "sre data.dta" file
clear all
version 12
capture mkdir "${dir}\sre output\"
global output "${dir}\sre output\"

****************************************************************************************************
*Hypergamy ratios 

net search st0085_2  // User: These commands make sure package to make regression tables is installed
net install st0085_2 //       and install it if needed

****************************************************************************************************
cd "${dir}"
use "sre data.dta", clear
des
notes

*Quasi-symmetry model parameters
gen cr1=0
gen cr2=0
gen cr3=0
replace cr1=-.5 if (educm==1 & educf==2) | (educm==2 & educf==1)
replace cr2=-.5 if (educm==1 & educf==3) | (educm==3 & educf==1)
replace cr3=-.5 if (educm==2 & educf==3) | (educm==3 & educf==2)
lab def lmix 0 "WW" 1 "BB" 2 "MIX", replace
gen mix=.
replace mix=0 if racem==0 & racef==0
replace mix=1 if racem==1 & racef==1
replace mix=2 if (racem==1 & racef==0) | (racem==0 & racef==1)
lab val mix lmix
gen white=(mix==0)
gen black=(mix==1)
gen bwwb=(mix==2)
foreach g in cr1 cr2 cr3 {
gen ww`g'=(`g'*white)
gen bb`g'=(`g'*black)
gen mix`g'=(`g'*bwwb)
}
gen endog=-1
replace endog=0 if racem==racef
gen educ=((educm)+(educf))/2
gen endog_educ=endog*educ

*Model setup, run, and predict values
global cross wwcr1 wwcr2 wwcr3 bbcr1 bbcr2 bbcr3 mixcr1 mixcr2 mixcr3
global qs i.educm*i.racem	i.educf*i.racef	${cross} endog endog_educ
foreach yr in 1980 1990 2000 2010 {
eststo qmarry`yr': xi: glm freq $qs if year==`yr' & cohab==0, family(poisson) link(log)
predict m`yr' if e(sample)
eststo qcohab`yr': xi: glm freq $qs if year==`yr' & cohab==1, family(poisson) link(log)
predict c`yr' if e(sample)
}

*Table output of quasi-symmetry model
foreach g in marry cohab {
esttab q`g'1980 q`g'1990 q`g'2000 q`g'2010 using "${output}table a3 `g'.txt" ///
, b(3) se(3) nogaps compress label tab replace alignment(r) ///
mtitles("1980" "1990" "2000" "2010") lines nonotes ///
order(_Ieducm_2 _Ieducm_3 _Iracem_1 _IeduXrac_2_1 _IeduXrac_3_1 _Ieducf_2 _Ieducf_3 _Iracef_1 ///
_IeduXraca2_1 _IeduXraca3_1 wwcr1 bbcr1 mixcr1 wwcr2 bbcr2 mixcr2 wwcr3 bbcr3 mixcr3 endog endog_educ) ///
coeflabels(_Ieducm_2  "Husband some college (omitted category: HS)" ///
_Ieducm_3  "Husband college graduate (omitted category: HS)" ///
_Iracem_1  "BH (omitted category: white husband)" ///
_IeduXrac_2_1  "Husband's Some College X BH" ///
_IeduXrac_3_1  "Husband's College Graduate X BH" ///
_Ieducf_2  "Wife some college (omitted category: HS)" ///
_Ieducf_3  "Wife college graduate (omitted category: HS)" ///
_Iracef_1  "BW (omitted category: white wife)" ///
_IeduXraca2_1  "Wife's Some College X BW" ///
_IeduXraca3_1  "Wife's College Graduate X BW" ///
wwcr1  "HS vs. some college (WW)" ///
bbcr1  "HS vs. some college BB" ///
mixcr1  "HS vs. some college Whbl" ///
wwcr2  "HS vs. college graduate (WW)" ///
bbcr2  "HS vs. college graduate BB" ///
mixcr2  "HS vs. college graduate Whbl" ///
wwcr3  "Some college vs. college graduate (WW)" ///
bbcr3  "Some college vs. college graduate BB" ///
mixcr3  "Some college vs. college graduate WhBl" ///
endog  "Endogamy" ///
endog_educ "Endogamy X Mean Education Couple" ///
_cons "Constant")
}

*Construct hypergamy ratios (expected / observed)
gen expect=.
format expect %20.10f
foreach yr in 1980 1990 2000 2010 {
replace expect=m`yr' if year==`yr' & cohab==0
replace expect=c`yr' if year==`yr' & cohab==1
}
gen dominance=.
replace dominance=0 if educm==educf
replace dominance=1 if educm>educf
replace dominance=2 if educm<educf
label variable dominance "heterogamy gender dominance"
bysort cohab year type	dominance: egen obs=total(freq)
bysort cohab year type	dominance: egen exp=total(expect)
collapse (sum) observed=freq (sum) expected=expect, by(cohab year type dominance)
label define ldominance 0 "homogamy" 1 "male" 2 "female"		
lab values dominance ldominance
sort cohab year type dominance
order cohab year type dominance observed
lab var expected "predicted"
bysort cohab year type (dominance): egen totobs=total(observed)
bysort cohab year type (dominance): egen totexp=total(expected)
gen phomogobs=observed/totobs if dominance==0
gen pmaleobs=observed/totobs if dominance==1
gen pfemaleobs=observed/totobs if dominance==2
gen phomogexp=expected/totexp if dominance==0
gen pmaleexp=expected/totexp if dominance==1
gen pfemaleexp=expected/totexp if dominance==2
foreach g in homog male female {
bysort cohab year type (dominance): egen obs`g'=max(p`g'obs)
bysort cohab year type (dominance): egen exp`g'=max(p`g'exp)
} // these are proportions of total within each dominance type
collapse (firstnm) obshomog exphomog obsmale expmale ///
obsfemale expfemale observed expected, by(cohab year type)
gen obsratio=obsmale/obsfemale
gen expratio=expmale/expfemale
gen ratio=obsratio/expratio
lab var ratio "male dominance ratio"
gen hr = ratio
replace hr = 1/ratio if type==2
lab var hr "white hypergamy ratio"
sort cohab year type
order cohab year type hr
keep cohab year type hr
outsheet using "${output}hypergamy ratios.txt", replace

*Figure output
lab def rcohab 0 "A. Married couples" 1 "B. Cohabiting couples"
lab val cohab rcohab
twoway ///
(connected hr year if type==1, lwidth(medthick) lpattern(solid) lcolor(gs0) msymbol(O) mcolor(gs0) msize(2)) ///
(connected hr year if type==2, lwidth(medthick) lpattern(solid) lcolor(gs0) msymbol(X) mcolor(gs0) msize(3)) ///
(connected hr year if type==3, lwidth(medthick) lpattern(dash) lcolor(gs0) msymbol(Dh) mcolor(gs0) msize(2)) ///
(connected hr year if type==4, lwidth(medthick) lpattern(dash) lcolor(gs0) msymbol(Th) mcolor(gs0) msize(2)) ///
, ///
by(cohab, note("") imargin(medium) iyaxes iylabel ixaxes graphregion(color(white)) plotregion(color(white))) ///
ytitle("Hypergamy ratio", size(3.2)) ///
yscale(range(1 1.8)) ///
ylabel(1(.2)1.8, labsize(3) angle(0) glcolor(gs13) gmax glwidth(vthin)) ///
xtitle("Year", color(white) size(4)) ///
xscale(range(1980 2010)) ///
xlabel(1980 1990 2000 2010, labsize(3.2)) ///
subtitle(,fcolor(white) lcolor(white) size(3.2) ring(1) pos(12)) ///
text(.8 1995 "Year", size(3.2)) ///
legend( stack order(1 "White male/white female" 2 "White male/black female" 3 "Black male/white female" 4 "Black male/black female") ///
size(2.8) ring(0) row(1) symx(12) symy(3) lcolor(white) symplacement(center) ) ///
xsize(5.5) ysize(3.5)
graph export "${output}figure 4 hypergamy ratios.tif", replace width(3300)
window manage close graph
				
****************************************************************************************************
*Dyadic Exchange and Educational Boundaries
****************************************************************************************************
cd "${dir}"
use "sre data.dta", clear
des
notes

*Status exchange parameters
*BW couples
gen sebmwfup=0
replace sebmwfup=1 if type==3 & educm>educf
gen sebmwfdown=0
replace sebmwfdown=1 if type==3 & educf>educm
*WB couples
gen sewmbfup=0
replace sewmbfup=1 if type==2 & educm<educf
gen sewmbfdown=0
replace sewmbfdown=1 if type==2 & educm>educf
*WW couples
gen sewmwfup=0
replace sewmwfup=1 if type==1 & educm<educf
gen sewmwfdown=0
replace sewmwfdown=1 if type==1 & educm>educf
*BB couples
gen sebmbfup=0
replace sebmbfup=1 if type==4 & educm<educf
gen sebmbfdown=0
replace sebmbfdown=1 if type==4 & educm>educf
*Educational boundary parameters
*BW couples
gen ebbm1=0
replace ebbm1=1 if educm>1 & type==3
gen ebbm2=0
replace ebbm2=1 if educm>2 & type==3
gen ebwf1=0
replace ebwf1=1 if educf>1 & type==3
gen ebwf2=0
replace ebwf2=1 if educf>2 & type==3
*WB couples
gen ebwm1=0
replace ebwm1=1 if educm>1 & type==2
gen ebwm2=0
replace ebwm2=1 if educm>2 & type==2
gen ebbf1=0
replace ebbf1=1 if educf>1 & type==2
gen ebbf2=0
replace ebbf2=1 if educf>2 & type==2

*Model macros
global base i.racem*i.racef i.racem*i.educm i.racef*i.educf
global eam i.educm*i.educf
global se sebmwfup sebmwfdown sewmbfup sewmbfdown
global eb ebbm1 ebbm2 ebwf1 ebwf2 ebwm1 ebwm2 ebbf1 ebbf2
*Model setup
global a glm freq $base $eam $se
global b glm freq $base $eam $se $eb
global models m1 m2 m3
*Model run
foreach yr in 1980 1990 2000 2010 {
eststo a`yr'_marry: xi: $a if year==`yr' & cohab==0, family(poisson) link(log)
eststo b`yr'_marry: xi: $b if year==`yr' & cohab==0, family(poisson) link(log)
eststo a`yr'_cohab: xi: $a if year==`yr' & cohab==1, family(poisson) link(log)
eststo b`yr'_cohab: xi: $b if year==`yr' & cohab==1, family(poisson) link(log)
}

*Table output
foreach g in marry cohab {
esttab a1980_`g' b1980_`g' a1990_`g' b1990_`g' a2000_`g' b2000_`g' a2010_`g' b2010_`g' ///
using "${output}table a4 `g'.txt", csv replace b(3) se(3) nogaps nodepvars compress ///
alignment(r) nonotes noomitted mtitle("1980a" "1980b" "1990a" "1990b" "2000a" "2000b" "2010a" "2010b") ///
lines addnotes("Young couples age 25-34" "Coupletype `g'") ///
scalars(df deviance "ll log likelihood" aic bic) sfmt(%9.0f %16.2f) constant noobs nonumbers ///
order(sebmwfup sebmwfdown ebbm1 ebbm2 ebwf1 ebwf2 sewmbfup sewmbfdown ebbf1 ebbf2 ebwm1 ebwm2 ///
_Ieducm_2 _Ieducm_3 _Iracem_1 _IracXedu_1_2 _IracXedu_1_3 _Ieducf_2 _Ieducf_3	_Iracef_1	///
_IracXedua1_2 _IracXedua1_3 _IeduXedu_2_2 _IeduXedu_2_3 _IeduXedu_3_2 _IeduXedu_3_3 _IracXrac_1_1) ///
coeflabels(sebmwfup "White female hypergamy" ///
sebmwfdown "White female hypogamy" ///
ebbm1 "   Black male Some college (SC)" ///
ebbm2 "   Black male College grad (CG)" ///
ebwf1 "   White female SC" ///
ebwf2 "   White female CG" ///
sewmbfup "White male hypergamy" ///
sewmbfdown "White male hypogamy" ///
ebbf1 "   Black female SC" ///
ebbf2 "   Black female CG" ///
ebwm1 "   White male SC" ///
ebwm2 "   White male CG" ///
_Ieducm_2 " Male SC" ///
_Ieducm_3 " Male CG" ///
_Iracem_1 " Black male " ///
_IracXedu_1_2 " Black male * SC" ///
_IracXedu_1_3 " Black male * CG" ///
_Ieducf_2 " Female SC" ///
_Ieducf_3 " Female CG" ///
_Iracef_1 " Black female" ///
_IracXedua1_2 " Black female * SC" ///
_IracXedua1_3 " Black female * CG" ///
_IeduXedu_2_2 " Male SC Female SC" ///
_IeduXedu_2_3 " Male SC Female CG" ///
_IeduXedu_3_2 " Male CG Female SC" ///
_IeduXedu_3_3 " Male CG Female CG" ///
_IracXrac_1_1 " Black male * Black female")
}

*Figure output (a brute force version of coeffplot)
cd "${output}"
foreach g in marry cohab {
import delimited "table a4 `g'.txt", delimiter(`","= "') varnames(1) stripquote(yes) clear
drop if _n==1
drop v1 v3 v4 v6 v7 v9 v10 v12 v13 v15 v16 v18 v19 v21 v22 v24 v25 v27 
gen temp1=(v2!="")
gen temp2=sum(temp1)
bysort temp2: gen x=_n // 1=coeff 2=se
drop if temp2>=13
drop if temp2>=7 // black male / white female only
rename a a1980
rename b b1980
rename v11 a1990
rename v14 b1990
rename v17 a2000
rename v20 b2000
rename v23 a2010
rename v26 b2010
replace v2=v2[_n-1] if x==2
lab def lparam 1 "White hypergamy" 2 "White hypogamy" ///
3 "Black male, some college" 4 "Black male, college grad" ///
5 "White female, some college" 6 "White female, college grad", replace
gen param=temp2
lab val param lparam
drop v2 temp1 temp2
reshape long a b, i(param x) j(year)
reshape wide a b, i(param year) j(x)
destring a1, replace ignore("*")
destring b1, replace ignore("*")
destring a2, replace ignore("(" ")")
destring b2, replace ignore("(" ")")
gen upper1=a1+1.96*a2
gen lower1=a1-1.96*a2
gen upper2=b1+1.96*b2
gen lower2=b1-1.96*b2
drop a2 b2
rename a1 point1
rename b1 point2
reshape long point lower upper, i(param year) j(model)
lab def lmodel 1 "total" 2 "net_eb", replace
lab val model lmodel
*Exponentiate
replace point=exp(point)
replace upper=exp(upper)
replace lower=exp(lower)
*Dyadic parameters
gen help=param
order param help
gen xaxis=.
replace xaxis=2 if param==1 & model==1
replace xaxis=4 if param==1 & model==2
replace xaxis=7 if param==2 & model==1
replace xaxis=9 if param==2 & model==2
*Educational boundary parameters
gen educ=.
replace educ=1 if inlist(help,3,5)
replace educ=2 if inlist(help,4,6)
lab def leduc 1 "Some college" 2 "College grad", replace
lab val educ leduc
gen group=.
replace group=1 if inlist(help,3,4)
replace group=2 if inlist(help,5,6)
lab def lgroup 1 "Black males" 2 "White females", replace
lab val group lgroup
replace group=. if model==1
replace educ=. if model==1
sort group year educ
bysort group (year educ): gen xaxis2=_n if educ!=.
recode xaxis2 8=19 7=17 6=14 5=12 4=9 3=7 2=4 1=2
*Dyadic graph (scaled properly for married)
twoway ///
(rcap lower upper xaxis if lower>.8 & upper<2, lcolor(gs0) lwidth(.3)) ///
(scatter point xaxis if lower>.8 & upper<2, mcolor(gs0) msize(2.2)) ///
, by(year, row(1) iyaxes graphregion(color(white)) plotregion(color(white)) ///
legend(off) note("") iylabels iscale(1)) ///
xlabel(2 "Total" 3 "White hypergamy" 4 "Net of EB" 6 " " 7 "Total" 8 "White hypogamy" 9 "Net of EB" ///
, tlength(0) alternate labsize(3.2) labgap(2)) /// 
xmlabel(2 " " 4 " " 7 " " 9 " ", tlength(2) labsize(3.2)) /// 
xscale(range(1 10)) xtitle("") ///
ylabel(.85 1.0 1.2 1.4 1.6 1.8 2.0, angle(0) labsize(3.2) glwidth(thin) glcolor(gs13) gmax noticks) ///
yscale(range(.85 2) log) ytitle("Odds ratio", size(3.2)) ///
yline(1,lcolor(gs5) lwidth(medium)) ///
xsize(8) ysize(2.5) subtitle(,fcolor(white) lcolor(white) size(4) ring(1) pos(12) nobexpand)
graph export "figure 5a dyadic `g'.tif", replace width(4800)
window manage close graph
*Ed boundaries graph (scaled properly for married)
twoway ///
(rcap lower upper xaxis2 if lower>.65 & upper<2, lcolor(gs0) lwidth(.25)) ///
(scatter point xaxis2 if lower>.65 & upper<2, mcolor(gs0) msize(1.8)) ///
, by(group, row(1) iyaxes graphregion(color(white)) plotregion(color(white)) ///
legend(off) note("") iylabels iscale(1)) ///
xlabel(2 "Some college" 4 "College grad." 7 "Some college" 9 "College grad." /// 
12 "Some college" 14 "College grad." 17 "Some college" 19 "College grad.", tlength(0) alternate labsize(2.2) labgap(2)) /// 
xmlabel(2 " " 4 " " 7 " " 9 " " 12 " " 14 " " 17 " " 19 " ", tlength(2) labsize(1.8)) /// 
xscale(range(1 20)) xtitle("") ///
text(.55 3 "1980", size(2.8)) text(.55 8 "1990", size(2.8)) text(.55 13 "2000", size(2.8)) text(.55 18 "2010", size(2.8)) ///
ylabel(.65 .85 1.0 1.2 1.4 1.6 1.8 2.0, angle(0) labsize(2.8) glwidth(thin) glcolor(gs13) gmax noticks) ///
yscale(range(.65 2) log) ytitle("Odds ratio", size(2.8)) ///
yline(1,lcolor(gs5) lwidth(medium)) ///
xsize(7) ysize(3.5) subtitle(,fcolor(white) lcolor(white) size(3.2) ring(1) pos(12) nobexpand)
graph export "figure 5b educ `g'.tif", replace width(3600)
window manage close graph
*Dyadic graph (scaled properly for cohab)
twoway ///
(rcap lower upper xaxis if lower>.38 & upper<3.2, lcolor(gs0) lwidth(.3)) ///
(scatter point xaxis if lower>.38 & upper<3.2, mcolor(gs0) msize(2.2)) ///
, by(year, row(1) iyaxes graphregion(color(white)) plotregion(color(white)) ///
legend(off) note("") iylabels iscale(1)) ///
xlabel(2 "Total" 3 "White hypergamy" 4 "Net of EB" 6 " " 7 "Total" 8 "White hypogamy" 9 "Net of EB" ///
, tlength(0) alternate labsize(3.2) labgap(2)) /// 
xmlabel(2 " " 4 " " 7 " " 9 " ", tlength(2) labsize(3.2)) /// 
xscale(range(1 10)) xtitle("") ///
ylabel(.5 1 2 3, angle(0) labsize(3.2) glwidth(thin) glcolor(gs13) nogmax noticks) ///
yscale(range(.38 3.2) log) ytitle("Odds ratio", size(3.2)) /// ///
yline(1,lcolor(gs5) lwidth(medium)) ///
xsize(8) ysize(2.5) subtitle(,fcolor(white) lcolor(white) size(4) ring(1) pos(12) nobexpand)
graph export "figure 6a dyadic `g'.tif", replace width(4800)
window manage close graph
*Ed boundaries graph (scaled properly for cohab)
twoway ///
(rcap lower upper xaxis2 if lower>.33 & upper<4.5, lcolor(gs0) lwidth(.25)) ///
(scatter point xaxis2 if lower>.33 & upper<4.5, mcolor(gs0) msize(1.8)) ///
, by(group, row(1) iyaxes graphregion(color(white)) plotregion(color(white)) ///
legend(off) note("") iylabels iscale(1)) ///
xlabel(2 "Some college" 4 "College grad." 7 "Some college" 9 "College grad." /// 
12 "Some college" 14 "College grad." 17 "Some college" 19 "College grad.", tlength(0) alternate labsize(2.2) labgap(2)) /// 
xmlabel(2 " " 4 " " 7 " " 9 " " 12 " " 14 " " 17 " " 19 " ", tlength(2) labsize(1.8)) /// 
xscale(range(1 20)) xtitle("") ///
text(.22 3 "1980", size(2.8)) text(.22 8 "1990", size(2.8)) text(.22 13 "2000", size(2.8)) text(.22 18 "2010", size(2.8)) ///
ylabel(.5 1 2 3 4 5, angle(0) labsize(2.8) glwidth(thin) glcolor(gs13) gmax noticks) ///
yscale(range(.33 5) log) ytitle("Odds ratio", size(2.8)) ///
yline(1,lcolor(gs5) lwidth(medium)) ///
xsize(7) ysize(3.5) subtitle(,fcolor(white) lcolor(white) size(3.2) ring(1) pos(12) nobexpand)
graph export "figure 6b educ `g'.tif", replace width(3600)
window manage close graph
}

rm "figure 5a dyadic cohab.tif" // delete version with incorrect scale
rm "figure 5b educ cohab.tif" // delete version with incorrect scale
rm "figure 6a dyadic marry.tif" // delete version with incorrect scale
rm "figure 6b educ marry.tif" // delete version with incorrect scale

clear all
